/*
  PI4Symbols.cpp by Bo, OZ2M, www-rudius.net/oz2m

  PI4 - PharusIgnis4 - a digital modulation (MGM) for beacon purposes

  Made with Microsoft Visual C++ 2010 Express

*/

#include "stdafx.h"
#include <math.h>
#include <stdio.h>
#include <float.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <iostream>

using namespace std;

#define PI4Symbols 146
#define PI4MaxInfoLength 8

static const char PI4Chars[]="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ /";
static const char PI4Vector[PI4Symbols]={0,0,1,0,0,1,1,1,1,0,1,0,1,0,1,0,0,1,0,0,0,1,0,0,0,1,1,0,0,1,
                                         1,1,1,0,0,1,1,1,1,1,0,0,1,1,0,1,1,1,1,0,1,0,1,1,0,1,1,0,1,0,
                                         0,0,0,0,1,1,1,1,1,0,1,0,1,0,0,0,0,0,1,1,1,1,1,0,1,0,0,1,0,0,
                                         1,0,1,0,0,0,0,1,0,0,1,1,0,0,0,0,0,1,1,0,0,0,0,1,1,0,0,1,1,1,
                                         0,1,1,1,0,1,1,0,1,0,1,0,1,0,0,0,0,1,1,1,0,0,0,0,1,1};

void PrintData(char *title, char *dat)
{
	int count,i;

  printf("%s (4x30+26)\n",title);
  printf(" ");

  count=strlen(title)+9;
  for (i=0;i<count;i++)
   	printf("-");
  printf("\n");

	for (i=0;i<146;i++)
  {
    printf(" %d",dat[i]);
		if ((i+1)%30==0)
      printf("\n");
	}
  printf("\n\n");
}

char Parity(unsigned long int Value)
{
  int Even=0;
  int BitNo;

  for (BitNo=0;BitNo<=31;BitNo++)
    if (((Value >> BitNo) & 0x01) != 0)
      Even=1-Even;

  return Even;
}

char GetCharNo(const char *ValidChars, char *Ch)
{
  const char *ptrCh;

  ptrCh=strchr(ValidChars,*Ch);
  if (ptrCh==NULL)
  {
    printf(" %c is an invalid character, thus replaced with \" \"\n",*Ch);
    *Ch=' ';
    return(strchr(ValidChars,' ')-ValidChars);
  }
  else
    return(ptrCh-ValidChars);
}
  
void ToUpper(char* string )
{
  unsigned int i;

  for (i=0;i<strlen(string);i++)
     if(string[i]>=97 && string[i]<=122)         // Is it uppercase?
       string[i]-= 32;
}

int main( int argc, char *argv[], char *envp[])
{   
  char Msg[]="OZ7IGY";
  char MsgPadded[PI4MaxInfoLength+1];
  int I,J,BitNo;

  printf("\n");
	printf(" ************************************\n");
	printf(" * PI4 message to symbols generator *\n");
	printf(" * by Bo, OZ2M  www.rudius.net/oz2m *\n");
	printf(" ************************************\n");
  printf("\n");

  if(argc==1)
  {
    printf(" Missing message to encode\n\n");
    printf(" syntax: PI4Symbols.exe message (replace embedded <space> with _)\n\n");
    printf(" Message set to OZ7IGY\n\n");
  }
  else
  {
    sprintf(Msg,"%s",argv[1]);
    ToUpper(Msg);
  }

  // Eight char message
  sprintf(MsgPadded,"%-8s",Msg);

  // Source encoding
  unsigned long long int SourceEnc=0;
  
  for (I=0;I<PI4MaxInfoLength;I++)
    SourceEnc=SourceEnc*38+(long long) GetCharNo(PI4Chars,&MsgPadded[I]);

  printf("\n");
  printf(" Input message   Eight char message   Source encoding\n");
  printf(" -------------   ------------------   ---------------\n");
  printf(" %-15s \"%s\"           %llu\n",Msg,MsgPadded,SourceEnc);
  printf("\n");

  // Convolutional Encoding
  const unsigned long Poly1=0xF2D05351;
  const unsigned long Poly2=0xE4613C47;
  unsigned long N=0;
  char ConvEnc[146];

  I=0;
  for (J=0;J<PI4Symbols/2;J++)
  {
    N <<= 1;
    if ((SourceEnc & 0x20000000000LL) != 0)
      N |= 1;
    SourceEnc <<= 1;

    ConvEnc[I++]=Parity(N & Poly1);
    ConvEnc[I++]=Parity(N & Poly2);
  }
  
	PrintData(" Convolutional encoded data", ConvEnc);

  // Interleaving
  int P=0;
  int R=0;
  char Interleaved[PI4Symbols];

  for (I=0;I<=255;I++)
  {
    for (BitNo=0;BitNo<=7;BitNo++)
    {
      if (((I >> BitNo) & 0x01)==0x01)
        R |= 1 << (7-BitNo);
      else
        R &= ~(1 << (7-BitNo));
    }

    if ((P<PI4Symbols) && (R<PI4Symbols))
      Interleaved[R]=ConvEnc[P++];
  }

	PrintData(" Interleaved data", Interleaved);

  //  Merge with Sync Vector
  char Symbols[PI4Symbols];

  for (I=0;I<PI4Symbols;I++)
    Symbols[I]=PI4Vector[I] | (Interleaved[I] << 1);

	PrintData(" Symbols", Symbols);

  system("pause");                                       // Press any key to continue . . .
}